package com.supervision.minio.service.impl;

import com.supervision.common.domain.R;
import com.supervision.common.utils.StringUtils;
import com.supervision.minio.client.MinioTemplate;
import com.supervision.minio.domain.MinioFile;
import com.supervision.minio.mapper.MinioFileMapper;
import com.supervision.minio.service.MinioService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import jakarta.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.util.Date;
import java.util.Objects;
import java.util.UUID;

@Service
public class MinioServiceImpl implements MinioService {

    @Autowired
    private MinioTemplate minioTemplate;

    @Value("${minio.bucketName}")
    private String bucketName;

    @Autowired
    private MinioFileMapper minioFileMapper;

    @Override
    public String uploadFile(MultipartFile file) throws IOException {
        String suffix = Objects.requireNonNull(file.getOriginalFilename()).substring(file.getOriginalFilename().lastIndexOf(".") + 1);
        InputStream in = file.getInputStream();
        MinioFile minioFile = new MinioFile();
        minioFile.setFilename(file.getOriginalFilename());
        minioFile.setUploadDate(new Date());
        minioFile.setFileType(suffix);
        minioFile.setSize(in.available());
        int i = minioFileMapper.insert(minioFile);
        if (i > 0) {
            minioTemplate.putObject(bucketName, minioFile.getId(), null, in, in.available(), null);
            return minioFile.getId();
        } else {
            return null;
        }
    }

    @Override
    public void downloadFile(String fileId, HttpServletResponse response){
        MinioFile minioFile = minioFileMapper.selectById(fileId);
        String fileType = minioFile.getFileType();
        minioTemplate.getObject(bucketName, fileId, fileType, response);
    }

    @Override
    public R<?> delFile(String fileId) {
        MinioFile minioFile = minioFileMapper.selectById(fileId);
        minioFile.setDataStatus(StringUtils.getUUID());
        minioFileMapper.updateById(minioFile);
        minioTemplate.removeObject(bucketName, fileId);
        return R.ok();
    }

}
