package com.supervision.neo4j.repository;

import com.supervision.neo4j.domain.CaseNode;
import org.springframework.data.neo4j.core.schema.Relationship;
import org.springframework.data.neo4j.repository.Neo4jRepository;
import org.springframework.data.neo4j.repository.query.Query;
import org.springframework.data.repository.query.Param;

import java.util.List;

/**
 * @author qmy
 * @since 2023-10-26
 */
//@Repository
public interface CaseNodeRepository extends Neo4jRepository<CaseNode,Long> {

    @Query("MATCH p=(n:CaseNode) RETURN p")
    List<CaseNode> selectAll();

    @Query("MATCH(p:Person{name:$name}) return p")
    CaseNode findByName(@Param("name") String name);

    @Query("match (a)-[r:`:#{literal(#relation)}`]-(b) where a.name = $fromNodeName and b.name = $toNodeName return r")
    Relationship queryRelationShip(@Param("fromNodeName") String fromNodeName, @Param("toNodeName") String toNodeName, @Param("relation") String relation);

    /**
     * 创建任意两个节点关系(fromNodeId -> relation -> toNodeId)
     * @param fromNodeId 源节点id
     * @param toNodeId 目标节点id
     * @param relation 关系
     */
    @Query("match (a),(b) where id(a)= $fromNodeId and id(b)= $toNodeId merge (a)-[:`:#{literal(#relation)}`]->(b)")
    void createRelationShip(@Param("fromNodeId") Long fromNodeId, @Param("toNodeId") Long toNodeId, @Param("relation") String relation);


    @Query("match (a),(b) where a.name = $fromNodeName and b.name= $toNodeName merge (a)-[:`:#{literal(#relation)}`]->(b)")
    void createRelationShip(@Param("fromNodeName") String fromNodeName, @Param("toNodeName") String toNodeName, @Param("relation") String relation);

}