package com.supervision.police.controller;

import com.supervision.common.domain.R;
import com.supervision.police.service.CaseEvidenceService;
import com.supervision.police.vo.CaseEvidenceReqVo;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;


/**
 * 案件证据控制器
 */
@Slf4j
@RestController
@Tag(name = "案件证据控")
@RequiredArgsConstructor
@RequestMapping("/caseEvidence")
public class CaseEvidenceController {

    private final CaseEvidenceService caseEvidenceService;

    @Operation(summary = "保存案件证据信息")
    @PostMapping("/saveEvidence")
    public R<String> saveEvidence(@RequestBody CaseEvidenceReqVo reqVO) {
        String evidenceId = caseEvidenceService.saveEvidence(reqVO);
        return R.ok(evidenceId);
    }

    @Operation(summary = "更新案件证据信息")
    @PostMapping("/updateEvidence")
    public R<String> updateEvidence(@RequestBody CaseEvidenceReqVo reqVO) {
        String evidenceId =  caseEvidenceService.updateEvidence(reqVO);
        return R.ok(evidenceId);
    }

    @Operation(summary = "删除案件证据信息")
    @DeleteMapping("/deleteEvidence")
    public R<Boolean> deleteEvidence(String evidenceId) {
        boolean success = caseEvidenceService.deleteEvidence(evidenceId);
        return R.ok(success);
    }

    @Operation(summary = "查询案件证据信息列表")
    @GetMapping("/list")
    public void queryEvidenceList(@RequestParam @Parameter(name = "caseId",description = "案件id") String caseId) {
        caseEvidenceService.queryEvidenceList(caseId);
    }
}
