package com.supervision.police.controller;

import com.supervision.common.domain.R;
import com.supervision.police.domain.ComDictionary;
import com.supervision.police.dto.DictionaryByTypeParam;
import com.supervision.police.service.ComDictionaryService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author baomidou
 * @since 2022-03-16
 */
@RestController
@RequestMapping("/comDictionary")
@Api(tags = "字典表接口")
public class ComDictionaryController {

    @Autowired
    private ComDictionaryService dictionaryService;

    @GetMapping
    @ApiOperation(value = "查询字典", httpMethod = "GET")
    public R<?> findDictionary(ComDictionary dictionary,
                               @RequestParam(required = false, defaultValue = "1") Long page,
                               @RequestParam(required = false, defaultValue = "10") Long size) {
        return dictionaryService.findDictionaryInfo(dictionary, page, size);
    }

    @PostMapping("/queryByType")
    @ApiOperation(value = "通过type查询字典,可一次性查询多个", httpMethod = "POST")
    public R<?> findDictionaryListByType(@RequestBody @Validated DictionaryByTypeParam dictionaryByTypeParam) {
        return dictionaryService.findDictionaryListByType(dictionaryByTypeParam);
    }

    @PostMapping("/save")
    @ApiOperation(value = "新增字典", httpMethod = "POST")
    public R<?> insertDictionary(@RequestBody @Validated ComDictionary dictionary) {
        return dictionaryService.insertDictionary(dictionary);
    }

    @PostMapping("/update")
    @ApiOperation(value = "修改字典", httpMethod = "POST")
    public R<?> updateDictionary(@RequestBody ComDictionary dictionary) {
        return dictionaryService.updateDictionary(dictionary);
    }

    @PostMapping("/deleteByStatus/delete")
    @ApiOperation(value = "逻辑删除字典", httpMethod = "POST")
    public R<?> logicDeleteDictionary(@RequestParam Long id){
        return dictionaryService.logicDeleteDictionary(id);
    }

}
