package com.supervision.police.service.impl;

import cn.hutool.core.util.StrUtil;
import com.alibaba.druid.util.StringUtils;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.supervision.common.domain.R;
import com.supervision.neo4j.domain.CaseNode;
import com.supervision.neo4j.domain.Rel;
import com.supervision.neo4j.service.Neo4jService;
import com.supervision.police.domain.ModelRecordType;
import com.supervision.police.domain.NoteRecord;
import com.supervision.police.domain.NotePrompt;
import com.supervision.police.domain.TripleInfo;
import com.supervision.police.mapper.ModelRecordTypeMapper;
import com.supervision.police.mapper.NoteRecordMapper;
import com.supervision.police.mapper.NotePromptMapper;
import com.supervision.police.mapper.TripleInfoMapper;
import com.supervision.police.service.ModelRecordTypeService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.ai.chat.ChatResponse;
import org.springframework.ai.chat.Generation;
import org.springframework.ai.chat.messages.Message;
import org.springframework.ai.chat.messages.SystemMessage;
import org.springframework.ai.chat.messages.UserMessage;
import org.springframework.ai.chat.prompt.Prompt;
import org.springframework.ai.ollama.OllamaChatClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StopWatch;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Slf4j
@Service
@RequiredArgsConstructor
public class ModelRecordTypeServiceImpl extends ServiceImpl<ModelRecordTypeMapper, ModelRecordType> implements ModelRecordTypeService {

    private final ModelRecordTypeMapper modelRecordTypeMapper;

    private final NoteRecordMapper noteRecordMapper;

    private final NotePromptMapper notePromptMapper;

    private final TripleInfoMapper tripleInfoMapper;

    private final Neo4jService neo4jService;

    private final OllamaChatClient chatClient;

    @Override
    public List<ModelRecordType> queryType(String name, Integer page, Integer size) {
//        IPage<ModelRecordType> iPage = new Page<>(page, size);
//        iPage  = modelRecordTypeMapper.selectByName(iPage, name);
//        return R.ok(IPages.buildDataMap(iPage));
        List<ModelRecordType> list = modelRecordTypeMapper.selectByName(name);

        for (ModelRecordType modelRecordType : list) {
            //笔录内容
            List<NoteRecord> noteRecords = noteRecordMapper.selectByRecordType(modelRecordType.getRecordType());
            modelRecordType.setRecords(noteRecords);
            //提示词
            List<NotePrompt> prompts = notePromptMapper.queryPrompt(modelRecordType.getId());
            modelRecordType.setPrompts(prompts);
        }
        return list;
//        return R.ok(IPages.buildDataMap(iPage));
    }

    @Override
    public ModelRecordType queryByName(String content) {
        Wrapper<ModelRecordType> wrapper = new QueryWrapper<ModelRecordType>().eq("record_type", content);
        return modelRecordTypeMapper.selectOne(wrapper);
    }

    @Override
    public R<?> saveType(ModelRecordType type) {
        String id = type.getId();
        int i = 0;
        if (StringUtils.isEmpty(id)) {
            //新增
            i = modelRecordTypeMapper.insert(type);
        } else {
            i = modelRecordTypeMapper.updateById(type);
        }
        if (i > 0) {
            return R.ok("保存成功");
        } else {
            return R.fail("保存失败");
        }
    }

    @Override
    public R<?> addOrUpdPrompt(NotePrompt prompt) {
        int i = 0;
        if (StringUtils.isEmpty(prompt.getId())) {
            i = notePromptMapper.insert(prompt);
        } else {
            i = notePromptMapper.updateById(prompt);
        }
        if (i > 0) {
            return R.ok("保存成功");
        } else {
            return R.fail("保存失败");
        }
    }

    @Override
    public R<?> delPrompt(NotePrompt prompt) {
        String id = prompt.getId();
        int i = notePromptMapper.deleteById(id);
        if (i > 0) {
            return R.ok("删除成功");
        } else {
            return R.fail("删除失败");
        }
    }

    private List<TripleInfo> extractTripleInfo(String caseId, String name, String recordId) {
        List<NoteRecord> records = noteRecordMapper.selectRecord(caseId, name, recordId);
        List<TripleInfo> tripleInfos = new ArrayList<>();
        for (NoteRecord record : records) {
            List<NotePrompt> prompts = notePromptMapper.queryPrompt(record.getRecordTypeId());
            for (NotePrompt prompt : prompts) {
                if (StringUtils.isEmpty(prompt.getPrompt())) {
                    continue;
                }
                try {
                    StopWatch stopWatch = new StopWatch();
                    // 分析三元组
                    Prompt ask = new Prompt(new UserMessage(prompt.getPrompt() + record.getQuestion() + record.getAnswer()));
                    stopWatch.start();
                    log.info("开始分析:");
                    ChatResponse call = chatClient.call(ask);
                    stopWatch.stop();
                    log.info("耗时:{}", stopWatch.getTotalTimeSeconds());
                    String content = call.getResult().getOutput().getContent();
                    log.info("分析的结果是:{}", content);
                    JSONObject jsonObject = new JSONObject(content);
                    JSONArray threeInfo = jsonObject.getJSONArray("result");
                    for (int i = 0; i < threeInfo.length(); i++) {
                        JSONObject object = threeInfo.getJSONObject(i);
                        String startNodeType = object.getString("startNodeType");
                        String entity = object.getString("entity");
                        String endNodeType = object.getString("endNodeType");
                        String property = object.getString("property");
                        String value = object.getString("value");
                        // 去空,如果存在任何的空值,则忽略
                        if (StrUtil.hasEmpty(startNodeType, entity, endNodeType, property, value)) {
                            continue;
                        }
                        TripleInfo tripleInfo = new TripleInfo(entity, property, value, record.getId(), new Date(), startNodeType, endNodeType);
                        tripleInfoMapper.insert(tripleInfo);
                        tripleInfos.add(tripleInfo);
                    }
                } catch (Exception e) {
                    log.error(e.getMessage(), e);
                }
            }
        }
        return tripleInfos;
    }

    @Override
    public List<TripleInfo> getThreeInfo(String caseId, String name, String recordId) {
        // TODO 这里应该改成异步的形式,通过异步的形式来进行提取三元组信息,不能每次点击就跑一遍
        return extractTripleInfo(caseId, name, recordId);
    }

    @Override
    public String addNeo4j(List<String> ids) {
        List<TripleInfo> tripleInfos = tripleInfoMapper.selectByIds(ids);
        int i = 0;
        for (TripleInfo tripleInfo : tripleInfos) {
            try {
                //开始节点
                String start = tripleInfo.getStartNode();
                // 首先看是否已经存在了,如果已经存在了,就不添加了
                CaseNode startNode = neo4jService.findOneByName(tripleInfo.getCaseId(), tripleInfo.getNoteRecordsId(), tripleInfo.getStartNodeType(), start, "1");
                if (startNode == null) {
                    startNode = new CaseNode(start, tripleInfo.getStartNodeType(), tripleInfo.getNoteRecordId(), tripleInfo.getNoteRecordsId(), tripleInfo.getCaseId(), "1");
                    CaseNode save = (CaseNode) neo4jService.save(startNode).getData();
                    startNode.setId(save.getId());
                }
                //结束节点
                String end = tripleInfo.getEndNode();
                CaseNode endNode = neo4jService.findOneByName(tripleInfo.getCaseId(), tripleInfo.getNoteRecordsId(), tripleInfo.getEndNodeType(), end, "1");
                if (endNode == null) {
                    endNode = new CaseNode(end, tripleInfo.getEndNodeType(), tripleInfo.getNoteRecordId(), tripleInfo.getNoteRecordsId(), tripleInfo.getCaseId(), "1");
                    CaseNode save = (CaseNode) neo4jService.save(endNode).getData();
                    endNode.setId(save.getId());
                }
                //关系
                Rel rel = new Rel(startNode.getId(), tripleInfo.getRelation(), endNode.getId());
                Rel relation = neo4jService.findRelation(rel);
                if (relation == null) {
                    neo4jService.saveRelation(rel);
                }
                tripleInfo.setAddNeo4j("1");
                int j = tripleInfoMapper.updateById(tripleInfo);
                if (j > 0) {
                    i++;
                }
                // TODO 重复添加的OK了,删除的呢?
            } catch (Exception e) {
                log.error(e.getMessage(), e);
            }
        }
        return ("成功插入" + i + "条信息");
    }
}
