package com.supervision.springaidemo.controller;

import cn.hutool.json.JSONUtil;
import com.supervision.springaidemo.dto.AnswerDTO;
import com.supervision.springaidemo.dto.GlobalResult;
import com.supervision.springaidemo.util.TtsUtil;
import org.springframework.ai.chat.ChatResponse;
import org.springframework.ai.chat.Generation;
import org.springframework.ai.chat.messages.AssistantMessage;
import org.springframework.ai.chat.messages.Message;
import org.springframework.ai.chat.messages.SystemMessage;
import org.springframework.ai.chat.messages.UserMessage;
import org.springframework.ai.chat.prompt.Prompt;
import org.springframework.ai.ollama.OllamaChatClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;

@RequestMapping("virtual-patient")
@RestController
public class VirtualPatientTestController {

    private final OllamaChatClient chatClient;


    // 使用多线程进行提交


    @Autowired
    public VirtualPatientTestController(OllamaChatClient chatClient) {
        this.chatClient = chatClient;
    }

    public static final String userPromptTemplate = """
            预设病例的基本情况如下：
            ---
            名字:xxx
            年龄:xxx
            性别:女
            主诉:头疼，发烧。
            居住地:山东。
            职业:学生
            饮食:情况良好，胃口很大，总是感到饥饿，其他方面正常。
            大小便:小便正常，但每天有大约3次稀便。
            伴随症状:包括手抖、怕热、容易出汗，虽然胃口很好，吃很多却总是觉得饿，同时感到紧张、容易发脾气，经常拉肚子，体重有所下降。
            ---
            回复格式为json:{"answer":"根据病例情况回复的内容"}
            """;

    private static final String systemPrompt = """
            我们现在来进行一个角色扮演连续对话场景。
            你现在扮演一个病人的角色(小樊)，我来扮演医生,我们进行模拟问诊。
            我会给你设定病人的基本情况,你根据给定病人的基本信息进行回复我。
            如果参考信息中不能回答医生的问题，则回复'我不知道/没注意/不清楚'",所有问话都必须给回复!
            如果医生说'你好'等礼貌用语,你就回复'你好,医生'!!!
            """;

    private static final List<Message> messageHistoryList = new ArrayList<>();

    static {
        messageHistoryList.add(new SystemMessage(systemPrompt));
        messageHistoryList.add(new UserMessage(userPromptTemplate));
        messageHistoryList.add(new AssistantMessage("好的,已了解我要扮演病人的基本情况。已准备好对话了。"));
    }

    @GetMapping("talk")
    public String talk(String question) {
        messageHistoryList.add(new UserMessage(question));
        Prompt prompt = new Prompt(messageHistoryList);
        ChatResponse call = chatClient.call(prompt);
        Generation result = call.getResult();
        String content = result.getOutput().getContent();
        messageHistoryList.add(new AssistantMessage(content));
        return content;
    }

    @GetMapping("chatVoice")
    public GlobalResult<AnswerDTO> chatVoice(String question){
        String talk = talk(question);
        AnswerDTO answer = JSONUtil.toBean(talk, AnswerDTO.class);
        String answerBase64 = TtsUtil.ttsTransform(answer.getAnswer());
        answer.setAnswerBase64(answerBase64);
        return GlobalResult.ok(answer);
    }

}
